@@ -22,7 +22,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #endif
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "metadata-widget.h"
 #include "common-defs.h"
 #include <gtk/gtk.h>
@@ -44,7 +44,7 @@ struct _MetadataWidgetPrivate
   GtkWidget* piece_label;
   GtkWidget* container_label;
   GtkWidget* player_label;
-  GdkPixbuf* icon_buf;
+  GtkWidget* player_icon;
   DbusmenuMenuitem* twin_item;
   gint		 current_height;
 };
@@ -126,10 +126,18 @@ metadata_widget_init (MetadataWidget *self)
   MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(self);
   GtkWidget *hbox;
   GtkWidget *outer_v_box;
-  priv->icon_buf = NULL;
-    
-  outer_v_box = gtk_vbox_new (FALSE, 0);
+  
+  #if GTK_CHECK_VERSION(3, 0, 0)  
+  outer_v_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  #else
+  outer_v_box = gtk_vbox_new (FALSE, 0);  
+  #endif
+ 
+  #if GTK_CHECK_VERSION(3, 0, 0)  
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  #else
   hbox = gtk_hbox_new(FALSE, 0);
+  #endif 
   
   priv->meta_data_h_box = hbox;
   priv->current_height = 1;
@@ -162,7 +170,12 @@ metadata_widget_init (MetadataWidget *self)
                       FALSE,
                       1); 
   priv->theme_change_occured = FALSE;
+
+  #if GTK_CHECK_VERSION(3, 0, 0)  
+  GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+  #else
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
+  #endif
   
   // artist
   GtkWidget* artist;
@@ -209,18 +222,36 @@ metadata_widget_init (MetadataWidget *self)
                    G_CALLBACK(metadata_widget_selection_received_event_callback),
                    GTK_WIDGET(self));   
 
+  gint padding = 4;
+  gtk_widget_style_get(GTK_WIDGET(self), "toggle-spacing", &padding, NULL);
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+  GtkWidget * tophbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, padding);
+#else
+  GtkWidget * tophbox = gtk_hbox_new(FALSE, padding);
+#endif
+
+  GtkWidget *player_icon;
+  player_icon = gtk_image_new();
+  priv->player_icon =  player_icon;
+
+  gtk_misc_set_alignment (GTK_MISC(priv->player_icon), 1.0 /* right aligned */, 0.5);
+  gtk_box_pack_start (GTK_BOX (tophbox), priv->player_icon, FALSE, FALSE, 0);
+  GtkWidget* spacer;
+  spacer = gtk_alignment_new (0,0,0,0);
+  gtk_container_add (GTK_CONTAINER (spacer), priv->meta_data_h_box);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (spacer),5,0,0,0);
+
   // player label
   GtkWidget* player_label;
-  player_label = gtk_label_new("");
-  gtk_misc_set_alignment(GTK_MISC(player_label), (gfloat)0, (gfloat)0.5);
-  gtk_misc_set_padding (GTK_MISC(player_label), (gfloat)1, (gfloat)0);  
-  gtk_widget_set_size_request (player_label, 200, 24);
+  player_label = gtk_label_new ("");
+  gtk_misc_set_alignment(GTK_MISC(player_label), (gfloat)0, 0.5);
   priv->player_label = player_label;
+  gtk_box_pack_start (GTK_BOX (tophbox), priv->player_label, TRUE, TRUE, 0);
       
-  gtk_box_pack_start (GTK_BOX(outer_v_box), priv->player_label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(outer_v_box), tophbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(outer_v_box), spacer, FALSE, FALSE, 0);
   
-  gtk_box_pack_start (GTK_BOX(outer_v_box), priv->meta_data_h_box, FALSE, FALSE, 0);
-    
   gtk_container_add (GTK_CONTAINER (self), outer_v_box);  
   
   gtk_widget_show_all (priv->meta_data_h_box);
@@ -232,20 +263,31 @@ metadata_widget_init (MetadataWidget *self)
 static void
 metadata_widget_dispose (GObject *object)
 {
-  MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(METADATA_WIDGET(object)); 
-
-  if (priv->icon_buf != NULL){
-    gdk_pixbuf_unref(priv->icon_buf);
-  }
   G_OBJECT_CLASS (metadata_widget_parent_class)->dispose (object);
 }
 
 static void
 metadata_widget_finalize (GObject *object)
 {
+  MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(METADATA_WIDGET(object)); 
+  g_string_free (priv->image_path, TRUE);
+  g_string_free (priv->old_image_path, TRUE);
+
   G_OBJECT_CLASS (metadata_widget_parent_class)->finalize (object);
 }
 
+/**
+* Make sure to only clear the album art only when it is not empty
+* Otherwise it will continuously call queue_draw after each empty call.
+*/
+static void 
+clear_album_art (GtkImage* album_art)
+{
+  if (gtk_image_get_storage_type(album_art) != GTK_IMAGE_EMPTY){  
+    gtk_image_clear (album_art);  
+  }  
+}
+
 
 #if GTK_CHECK_VERSION(3, 0, 0)  
 static void
@@ -255,9 +297,10 @@ metadata_widget_get_preferred_width (GtkWidget* self,
 {
   *minimum_width = *natural_width =  200;
 }
+
 /**
  * We override the expose method to enable primitive drawing of the 
- * empty album art image and rounded rectangles on the album art.
+ * empty album art image.
  */
 static gboolean
 metadata_image_expose_gtk_3 (GtkWidget *metadata,
@@ -274,18 +317,18 @@ metadata_image_expose_gtk_3 (GtkWidget *metadata,
     return FALSE;
   }
   
-  draw_album_border (metadata, FALSE);
+  if((guint)priv->image_path->len != 0){
 
-  if(priv->image_path->len > 0){
-    if(g_string_equal(priv->image_path, priv->old_image_path) == FALSE ||
+    if(g_string_equal (priv->image_path, priv->old_image_path) == FALSE ||
        priv->theme_change_occured == TRUE){
       priv->theme_change_occured = FALSE;
       GdkPixbuf* pixbuf;
       pixbuf = gdk_pixbuf_new_from_file_at_size(priv->image_path->str, 60, 60, NULL);
-
+      
       if(GDK_IS_PIXBUF(pixbuf) == FALSE){
-        gtk_image_clear ( GTK_IMAGE(priv->album_art));          
+        clear_album_art (GTK_IMAGE(priv->album_art));          
         gtk_widget_set_size_request(GTK_WIDGET(priv->album_art), 60, 60);
+        draw_album_border (metadata, FALSE);  
         draw_album_art_placeholder(metadata);
         return FALSE;
       }
@@ -295,14 +338,17 @@ metadata_image_expose_gtk_3 (GtkWidget *metadata,
                                   gdk_pixbuf_get_width(pixbuf),
                                   gdk_pixbuf_get_height(pixbuf));
 
+      draw_album_border (metadata, FALSE);  
       g_string_erase (priv->old_image_path, 0, -1);
       g_string_overwrite (priv->old_image_path, 0, priv->image_path->str);
       g_object_unref(pixbuf);
     }
     return FALSE;       
   }
-  gtk_image_clear (GTK_IMAGE(priv->album_art));  
+  clear_album_art (GTK_IMAGE(priv->album_art));  
+  g_string_erase (priv->old_image_path, 0, -1);  
   gtk_widget_set_size_request(GTK_WIDGET(priv->album_art), 60, 60);
+  draw_album_border (metadata, FALSE);
   draw_album_art_placeholder(metadata);
   return FALSE;
 }
@@ -310,14 +356,19 @@ metadata_image_expose_gtk_3 (GtkWidget *metadata,
 // Draw the triangle if the player is running ...
 static gboolean
 metadata_widget_icon_triangle_draw_cb_gtk_3 (GtkWidget *widget,
-											 cairo_t* cr,
-											 gpointer user_data)
-{
-	
-  g_return_val_if_fail(IS_METADATA_WIDGET(user_data), FALSE);
-  MetadataWidget* meta = METADATA_WIDGET(user_data);
+                      											 cairo_t* cr,
+                      											 gpointer user_data)
+{	
+  g_return_val_if_fail (IS_METADATA_WIDGET (user_data), FALSE);
+  MetadataWidget* meta = METADATA_WIDGET (user_data);
   MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(meta);  
 
+  gboolean running = dbusmenu_menuitem_property_get_bool (priv->twin_item,
+                                                          DBUSMENU_METADATA_MENUITEM_PLAYER_RUNNING);
+
+  if (!running)
+    return FALSE;
+
   GtkStyle *style;
   int x, y, arrow_width, arrow_height;
 
@@ -331,34 +382,18 @@ metadata_widget_icon_triangle_draw_cb_gtk_3 (GtkWidget *widget,
   x = allocation.x;
   y = 0;
 
-  gint offset = (allocation.height - gdk_pixbuf_get_height (priv->icon_buf)) / 2;
-
-  // Draw player icon  
-  if (priv->icon_buf != NULL){  
-    gdk_cairo_set_source_pixbuf (cr,
-                                 priv->icon_buf,
-                                 x + arrow_width + 1,
-                                 y + offset);
-    cairo_paint (cr);
-  }
-
   // Draw triangle but only if the player is running.
-  if (dbusmenu_menuitem_property_get_bool (priv->twin_item,
-                                             DBUSMENU_METADATA_MENUITEM_PLAYER_RUNNING)){
-    y += allocation.height/2.0 - (double)arrow_height/2.0;
-    cairo_set_line_width (cr, 1.0);
+  y += gtk_image_get_pixel_size (GTK_IMAGE (priv->player_icon)) / 3 + 5;
+  cairo_set_line_width (cr, 1.0);
 
-	//g_debug ("triangle drawing");
-
-    cairo_move_to (cr, x, y);
-    cairo_line_to (cr, x, y + arrow_height);
-    cairo_line_to (cr, x + arrow_width, y + (double)arrow_height/2.0);
-    cairo_close_path (cr);
-    cairo_set_source_rgb (cr, style->fg[gtk_widget_get_state(widget)].red/65535.0,
-                              style->fg[gtk_widget_get_state(widget)].green/65535.0,
-                              style->fg[gtk_widget_get_state(widget)].blue/65535.0);
-    cairo_fill (cr);                                             
-  }
+  cairo_move_to (cr, x, y);
+  cairo_line_to (cr, x, y + arrow_height);
+  cairo_line_to (cr, x + arrow_width, y + (double)arrow_height/2.0);
+  cairo_close_path (cr);
+  cairo_set_source_rgb (cr, style->fg[gtk_widget_get_state(widget)].red/65535.0,
+                            style->fg[gtk_widget_get_state(widget)].green/65535.0,
+                            style->fg[gtk_widget_get_state(widget)].blue/65535.0);
+  cairo_fill (cr);                                             
   
   return FALSE;  
 }
@@ -391,7 +426,7 @@ metadata_image_expose (GtkWidget *metadata,
       pixbuf = gdk_pixbuf_new_from_file_at_size(priv->image_path->str, 60, 60, NULL);
 
       if(GDK_IS_PIXBUF(pixbuf) == FALSE){
-        gtk_image_clear ( GTK_IMAGE(priv->album_art));          
+        clear_album_art (GTK_IMAGE(priv->album_art));          
         gtk_widget_set_size_request(GTK_WIDGET(priv->album_art), 60, 60);
         draw_album_art_placeholder(metadata);
         return FALSE;
@@ -408,7 +443,8 @@ metadata_image_expose (GtkWidget *metadata,
     }
     return FALSE;       
   }
-  gtk_image_clear (GTK_IMAGE(priv->album_art));  
+  clear_album_art (GTK_IMAGE(priv->album_art));  
+  g_string_erase (priv->old_image_path, 0, -1);      
   gtk_widget_set_size_request(GTK_WIDGET(priv->album_art), 60, 60);
   draw_album_art_placeholder(metadata);
   return FALSE;
@@ -422,9 +458,16 @@ metadata_widget_icon_triangle_draw_cb (GtkWidget *widget,
                                        gpointer user_data)
 {
   g_return_val_if_fail(IS_METADATA_WIDGET(user_data), FALSE);
+
   MetadataWidget* meta = METADATA_WIDGET(user_data);
   MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(meta);  
 
+  gboolean running = dbusmenu_menuitem_property_get_bool (priv->twin_item,
+                                                          DBUSMENU_METADATA_MENUITEM_PLAYER_RUNNING);
+
+  if (!running)
+    return FALSE;
+
   GtkStyle *style;
   cairo_t *cr;
   int x, y, arrow_width, arrow_height;
@@ -441,32 +484,18 @@ metadata_widget_icon_triangle_draw_cb (GtkWidget *widget,
   x = allocation.x;
   y = allocation.y;
 
-  gint offset = (allocation.height - gdk_pixbuf_get_height (priv->icon_buf)) / 2;
-
-  // Draw player icon  
-  if (priv->icon_buf != NULL){  
-    gdk_cairo_set_source_pixbuf (cr,
-                                 priv->icon_buf,
-                                 x + arrow_width + 1,
-                                 y + offset);
-    cairo_paint (cr);
-  }
-
   // Draw triangle but only if the player is running.
-  if (dbusmenu_menuitem_property_get_bool (priv->twin_item,
-                                             DBUSMENU_METADATA_MENUITEM_PLAYER_RUNNING)){
-    y += allocation.height/2.0 - (double)arrow_height/2.0;
-    cairo_set_line_width (cr, 1.0);
+  y += allocation.height/2.0 - (double)arrow_height/2.0;
+  cairo_set_line_width (cr, 1.0);
 
-    cairo_move_to (cr, x, y);
-    cairo_line_to (cr, x, y + arrow_height);
-    cairo_line_to (cr, x + arrow_width, y + (double)arrow_height/2.0);
-    cairo_close_path (cr);
-    cairo_set_source_rgb (cr, style->fg[gtk_widget_get_state(widget)].red/65535.0,
-                              style->fg[gtk_widget_get_state(widget)].green/65535.0,
-                              style->fg[gtk_widget_get_state(widget)].blue/65535.0);
-    cairo_fill (cr);                                             
-  }
+  cairo_move_to (cr, x, y);
+  cairo_line_to (cr, x, y + arrow_height);
+  cairo_line_to (cr, x + arrow_width, y + (double)arrow_height/2.0);
+  cairo_close_path (cr);
+  cairo_set_source_rgb (cr, style->fg[gtk_widget_get_state(widget)].red/65535.0,
+                            style->fg[gtk_widget_get_state(widget)].green/65535.0,
+                            style->fg[gtk_widget_get_state(widget)].blue/65535.0);
+  cairo_fill (cr);                                             
   
   cairo_destroy (cr);
   return FALSE;  
@@ -474,7 +503,8 @@ metadata_widget_icon_triangle_draw_cb (GtkWidget *widget,
 #endif
 
 static void
-draw_album_border(GtkWidget *metadata, gboolean selected)
+draw_album_border(GtkWidget *metadata,
+                  gboolean selected)
 {
   cairo_t *cr;  
   cr = gdk_cairo_create (gtk_widget_get_window (metadata));
@@ -501,7 +531,7 @@ draw_album_border(GtkWidget *metadata, gboolean selected)
   bg_normal.g = style->bg[0].green/65535.0;
   bg_normal.b = style->bg[0].blue/65535.0;
 
-  gint state = selected ? 5 : 0;
+  const gint state = selected ? GTK_STATE_SELECTED : GTK_STATE_NORMAL;
   
   fg_normal.r = style->fg[state].red/65535.0;
   fg_normal.g = style->fg[state].green/65535.0;
@@ -570,7 +600,7 @@ draw_album_border(GtkWidget *metadata, gboolean selected)
 
 static void
 draw_album_art_placeholder(GtkWidget *metadata)
-{       
+{         
   cairo_t *cr;  
   cr = gdk_cairo_create (gtk_widget_get_window (metadata));
   GtkStyle *style;
@@ -699,7 +729,7 @@ metadata_widget_property_update(DbusmenuMenuitem* item, gchar* property,
   } 
   else if(g_ascii_strcasecmp(DBUSMENU_METADATA_MENUITEM_ARTURL, property) == 0){
     g_string_erase(priv->image_path, 0, -1);
-    g_string_overwrite(priv->image_path, 0, g_variant_get_string (value, NULL));
+    g_string_overwrite (priv->image_path, 0, g_variant_get_string (value, NULL));
     gtk_widget_queue_draw(GTK_WIDGET(mitem));
   }
   else if (g_ascii_strcasecmp (DBUSMENU_METADATA_MENUITEM_PLAYER_NAME, property) == 0){
@@ -726,7 +756,7 @@ metadata_widget_handle_resizing (MetadataWidget* self)
   else{
     gtk_widget_show (priv->meta_data_h_box);     
   }
-  gtk_widget_queue_draw(GTK_WIDGET(self));      
+  gtk_widget_queue_draw(GTK_WIDGET(self));
 }
 
 static void
@@ -746,28 +776,18 @@ metadata_widget_set_style(GtkWidget* metadata, GtkStyle* style)
   MetadataWidget* widg = METADATA_WIDGET(metadata);
   MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(widg); 
   priv->theme_change_occured = TRUE;    
-  gtk_widget_queue_draw(GTK_WIDGET(metadata));  
+  gtk_widget_queue_draw (GTK_WIDGET(metadata));  
 }
 
 static void 
 metadata_widget_set_icon (MetadataWidget *self)
 {
-  MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(self); 
+  MetadataWidgetPrivate * priv = METADATA_WIDGET_GET_PRIVATE(self);   
 
-  if (priv->icon_buf != NULL){
-    gdk_pixbuf_unref(priv->icon_buf);
-    priv->icon_buf = NULL;    
-  }
-  
-  gint padding = 0;
-  gtk_widget_style_get(GTK_WIDGET(self), "horizontal-padding", &padding, NULL);
-  gint width, height;
-  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
-  
   GString* banshee_string = g_string_new ( "banshee" );
-  GString* app_panel = g_string_new ( g_utf8_strdown (dbusmenu_menuitem_property_get(priv->twin_item, DBUSMENU_METADATA_MENUITEM_PLAYER_NAME),
-                                                     -1));
-  GdkPixbuf* icon_buf;
+  gchar * tmp = g_utf8_strdown (dbusmenu_menuitem_property_get(priv->twin_item, DBUSMENU_METADATA_MENUITEM_PLAYER_NAME), -1);
+  GString* app_panel = g_string_new (tmp);
+  g_free (tmp);
   
   // Banshee Special case!  
   // Not ideal but apparently we want the banshee icon to be the greyscale one
@@ -784,12 +804,9 @@ metadata_widget_set_icon (MetadataWidget *self)
                         dbusmenu_menuitem_property_get ( priv->twin_item,
                                                          DBUSMENU_METADATA_MENUITEM_PLAYER_ICON ));  
   }
-  icon_buf = gtk_icon_theme_load_icon ( gtk_icon_theme_get_default(),
-                                        app_panel->str,
-                                        (width > height) ? width : height,
-                                        GTK_ICON_LOOKUP_GENERIC_FALLBACK, 
-                                        NULL );  
-  priv->icon_buf = icon_buf;
+  
+  gtk_image_set_from_icon_name(GTK_IMAGE (priv->player_icon), app_panel->str, GTK_ICON_SIZE_MENU);
+
   g_string_free ( app_panel, TRUE);
   g_string_free ( banshee_string, TRUE);
 }
