@@ -78,10 +78,8 @@ sound_state_manager_init (SoundStateManager* self)
   priv->notification = NULL;  
   priv->settings_manager = NULL;
 
-  priv->settings_manager = g_settings_new("com.canonical.indicators.sound");
+  priv->settings_manager = g_settings_new("com.canonical.indicator.sound");
 
-  sound_state_manager_notification_init (self);
-  
   sound_state_manager_prepare_state_image_names (self);
   sound_state_manager_prepare_blocked_animation (self);
 
@@ -134,6 +132,13 @@ sound_state_manager_class_init (SoundStateManagerClass *klass)
 static void
 sound_state_manager_notification_init (SoundStateManager* self)
 {
+  static gboolean initialized = FALSE;
+
+  /* one-time lazy initialization */
+  if (initialized)
+    return;
+  initialized = TRUE;
+
   SoundStateManagerPrivate* priv = SOUND_STATE_MANAGER_GET_PRIVATE(self);
 
   if (!notify_init(PACKAGE_NAME))
@@ -164,6 +169,8 @@ sound_state_manager_show_notification (SoundStateManager *self,
 {
   SoundStateManagerPrivate* priv = SOUND_STATE_MANAGER_GET_PRIVATE(self);
 
+  sound_state_manager_notification_init (self);
+  
   if (priv->notification == NULL || 
       g_settings_get_boolean (priv->settings_manager, "show-notify-osd-on-scroll") == FALSE){
     return;
