@@ -269,6 +269,13 @@ transport_widget_dispose (GObject *object)
     }
   }
   #endif
+
+  TransportWidgetPrivate* priv = TRANSPORT_WIDGET_GET_PRIVATE(object);
+  if (priv->command_coordinates != NULL) {
+    g_hash_table_destroy (priv->command_coordinates);
+    priv->command_coordinates = NULL;
+  }
+
   G_OBJECT_CLASS (transport_widget_parent_class)->dispose (object);
 }
 
@@ -302,6 +309,7 @@ transport_widget_expose (GtkWidget *button, GdkEventExpose *event)
 gboolean
 transport_widget_is_selected ( TransportWidget* widget )
 {
+  g_return_val_if_fail (IS_TRANSPORT_WIDGET (widget), FALSE);
   TransportWidgetPrivate* priv = TRANSPORT_WIDGET_GET_PRIVATE(widget);
   return priv->has_focus;
 }
@@ -561,15 +569,15 @@ transport_widget_collision_detection ( gint x,
 {
   TransportAction event = TRANSPORT_ACTION_NO_ACTION;
   
-  if (x > 67 && x < 112
+  if (x > 57 && x < 102
       && y > 12 && y < 40){
     event = TRANSPORT_ACTION_PREVIOUS;
   }
-  else if (x > 111 && x < 153
+  else if (x > 101 && x < 143
            && y > 5 && y < 47){
     event = TRANSPORT_ACTION_PLAY_PAUSE;
   }
-  else if (x > 152 && x < 197
+  else if (x > 142 && x < 187
            && y > 12 && y < 40){
     event = TRANSPORT_ACTION_NEXT;
   }   
